<?php

namespace Drupal\input_pattern\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\file\Plugin\Field\FieldFormatter\GenericFileFormatter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Twig\Environment as Twig_Environment;

/**
 * Plugin implementation of the 'file' formatter.
 */
#[FieldFormatter(
  id: 'file_attributes',
  label: new TranslatableMarkup('Generic file attributes'),
  field_types: [
    'file',
  ],
)]
class FileAttributesFormatter extends GenericFileFormatter {

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, protected FileUrlGeneratorInterface $fileUrlGenerator, protected Twig_Environment $twig) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('file_url_generator'),
      $container->get('twig'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'attributes' => '',
      'attached' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $element['attributes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom attributes'),
      '#description' => $this->t('Each variable is a new line. Can use with twig. Example: <br/>data-bs-toggle="tooltip"<br/>data-bs-title="{{entity.title.value}}"'),
      '#default_value' => $this->getSetting('attributes'),
    ];
    $element['attached'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Attached libraries'),
      '#description' => $this->t('Each library is a new line.'),
      '#default_value' => $this->getSetting('attached'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getSetting('attributes'))) {
      $summary[] = $this->t('Attributes: @attribute', [
        '@attribute' => $this->getSetting('attributes'),
      ]);
    }
    if (!empty($this->getSetting('attached'))) {
      $summary[] = $this->t('Libraries: @attached', [
        '@attached' => $this->getSetting('attached'),
      ]);
    }
    return array_merge($summary, parent::settingsSummary());
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);
    $settings = $this->getSettings();

    $attributes = [];
    if (!empty($settings['attributes'])) {
      $attributes = $this->extractAttributes($items, $settings['attributes']);
    }
    foreach ($attributes as $name => $value) {
      if ($name != 'class') {
        $elements['#attributes'][$name] = $value;
      }
      else {
        $elements['#attributes'][$name][] = $value;
      }
    }

    if (!empty($settings['attached'])) {
      $attached = explode(',', str_replace(PHP_EOL, ',', $settings['attached']));
      $elements['#attached']['library'] = [...$attached, ...($elements['#attached']['library'] ?? [])];
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  protected function extractAttributes($items, $settingAttr) {
    $attributes = [];
    $attrs = explode(PHP_EOL, $settingAttr);
    $context = [
      'entity' => $items->getEntity() ?? '',
      'field_name' => $items->getName(),
    ];
    foreach ($attrs as $attribute) {
      if (empty(trim($attribute))) {
        continue;
      }
      $variables = explode('=', $attribute);
      $key = trim(str_replace(['"', "'", ' '], ['', '', '-'], $variables[0]));
      $val = trim(str_replace('"', '', $variables[1] ?? $variables[0]));
      $attributes[$key] = $this->twig->renderInline($val, $context);
    }
    return $attributes;
  }

}
