<?php

namespace Drupal\input_pattern\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Twig\Environment as Twig_Environment;

/**
 * Plugin implementation of the 'string_attributes' formatter.
 */
#[FieldFormatter(
  id: 'list_attributes',
  label: new TranslatableMarkup('List attributes'),
  field_types: [
    'uri',
    'email',
    'string',
    'string_long',
  ],
)]
class ListAttributesFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, protected $entityTypeManager, protected EntityDisplayRepositoryInterface $entityDisplayRepository, protected Twig_Environment $twig) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('twig'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'list_type' => 'ul',
      'wrapper_attributes' => '',
      'attributes' => '',
      'attached' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);
    $element['list_type'] = [
      '#title' => $this->t('Style options'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('list_type'),
      '#options' => [
        'ul' => $this->t('Unordered list'),
        'ol' => $this->t('Ordered list'),
      ],
    ];
    $element['wrapper_attributes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Wrapper HTML element'),
      '#description' => $this->t('Each variable is a new line. Can use with twig.'),
      '#default_value' => $this->getSetting('wrapper_attributes'),
    ];
    $element['attributes'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Customize field HTML'),
      '#description' => $this->t('Each variable is a new line. Can use with twig.'),
      '#default_value' => $this->getSetting('attributes'),
    ];
    $element['attached'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Attached libraries'),
      '#description' => $this->t('Each library is a new line.'),
      '#default_value' => $this->getSetting('attached'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getSetting('wrapper_attributes'))) {
      $summary[] = $this->t('Wrapper attributes: @attribute', [
        '@attribute' => $this->getSetting('wrapper_attributes'),
      ]);
    }
    if (!empty($this->getSetting('attributes'))) {
      $summary[] = $this->t('Attributes: @attribute', [
        '@attribute' => $this->getSetting('attributes'),
      ]);
    }
    if (!empty($this->getSetting('attached'))) {
      $summary[] = $this->t('Libraries: @attached', [
        '@attached' => $this->getSetting('attached'),
      ]);
    }
    return array_merge($summary, parent::settingsSummary());
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $type = $items->getFieldDefinition()->getType();
    $entity = $items->getEntity();
    $entity_type = $entity->getEntityType();
    $settings = $this->getSettings();
    $display = $this->entityDisplayRepository->getViewDisplay($entity->getEntityTypeId(), $entity->bundle(), 'default')->getComponent($items->getName());
    $elements = [
      '#theme' => 'item_list',
      '#list_type' => $this->getSetting('list_type'),
      '#items' => [],
    ];
    if (!in_array($display['label'], ['hidden', 'visually_hidden'])) {
      $elements['#title'] = $items->getFieldDefinition()->getLabel();
    }

    $render_as_link = FALSE;
    if ($this->getSetting('link_to_entity') && !$entity->isNew() && $entity_type->hasLinkTemplate('canonical')) {
      $rel = $entity->getEntityType()->hasLinkTemplate('revision') ? 'revision' : 'canonical';
      $url = $entity->toUrl($rel);
      $access = $url->access(return_as_object: TRUE);
      (new CacheableMetadata())->addCacheableDependency($access)->applyTo($elements);
      $render_as_link = $access->isAllowed();
    }

    foreach ($items as $delta => $item) {
      if ($type == 'email') {
        $url = Url::fromUri('mailto:' . $item->value);
      }
      if ($type == 'uri') {
        $url = Url::fromUri($item->value);
      }
      if ($render_as_link) {
        assert(isset($url));
        $elements['#items'][$delta] = [
          '#type' => 'link',
          '#title' => $item->value,
          '#url' => $url,
        ];
      }
      else {
        $elements['#items'][$delta] = $item->value;
      }
    }
    if (!empty($settings['wrapper_attributes'])) {
      $attributes = $this->extractAttributes($items, $settings['wrapper_attributes']);
      foreach ($attributes as $name => $value) {
        if ($value != '') {
          if ($name != 'class') {
            $elements['#attributes'][$name] = $value;
          }
          else {
            $elements['#attributes'][$name][] = $value;
          }
        }
      }
    }
    if (!empty($settings['attributes'])) {
      $attributes = $this->extractAttributes($items, $settings['attributes']);
      foreach ($items as $delta => $item) {
        foreach ($attributes as $name => $value) {
          if (!empty($value)) {
            $elements['#items'][$delta] = [
              '#markup' => $item->value,
            ];
            if ($name != 'class') {
              $elements['#items'][$delta]['#wrapper_attributes'][$name] = $value;
            }
            else {
              $elements['#items'][$delta]['#wrapper_attributes'][$name][] = $value;
            }
          }
        }
      }
    }

    if (!empty($settings['attached'])) {
      $attached = explode(',', str_replace(PHP_EOL, ',', $settings['attached']));
      $elements['#attached']['library'] = [...$attached, ...($elements['#attached']['library'] ?? [])];
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  protected function extractAttributes($items, $settingAttr) {
    $attributes = [];
    $attrs = explode(PHP_EOL, $settingAttr);
    $context = [
      'entity' => $items->getEntity() ?? '',
      'field_name' => $items->getName(),
    ];
    foreach ($attrs as $attribute) {
      if (empty(trim($attribute))) {
        continue;
      }
      $variables = explode('=', $attribute);
      $key = trim(str_replace(['"', "'", ' '], ['', '', '-'], $variables[0]));
      $val = trim(str_replace('"', '', $variables[1] ?? $variables[0]));
      $attributes[$key] = $this->twig->renderInline($val, $context);
    }
    return $attributes;
  }

}
