<?php

namespace Drupal\input_pattern\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldFilteredMarkup;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Twig\Environment as Twig_Environment;

/**
 * Defines the 'input_pattern' field widget.
 */
#[FieldWidget(
  id: 'input_pattern',
  label: new TranslatableMarkup('Input attributes pattern'),
  field_types: [
    'integer',
    'decimal',
    'bigint',
    'float',
    'email',
    'string',
    'telephone',
  ],
)]
class InputPatternWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, protected AccountInterface $currentUser, protected Twig_Environment $twig) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('current_user'),
      $container->get('twig')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'type' => 'textfield',
      'placeholder' => '',
      'separator' => '',
      'maxlength' => '',
      'pattern' => '',
      'class' => '',
      'step' => '',
      'list' => '',
      'min' => '',
      'max' => '',
      'custom' => '',
      'use_auto_numeric' => FALSE,
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $settings = $this->getSettings();
    $field_settings = $this->getFieldSettings();
    $field_type = $this->fieldDefinition->getType();
    $field_name = $this->fieldDefinition->getName();
    $field_state = 'fields[' . $field_name . '][settings_edit_form][settings]';
    $base_step = $settings['base_step'] ?? 'any';
    $value_field_primitive = in_array($field_type, ['integer', 'float', 'decimal']) ? $this->getLimitNumber($field_settings, $field_type) : NULL;
    $min = $value_field_primitive && is_numeric($value_field_primitive['min']) ? $value_field_primitive['min'] : NULL;
    $max = $value_field_primitive && is_numeric($value_field_primitive['max']) ? $value_field_primitive['max'] : NULL;
    $element['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type for input'),
      '#options' => [
        'textfield' => $this->t('Text'),
        'number' => $this->t('Number'),
        'range' => $this->t('Range number'),
        'search' => $this->t('Search'),
        'tel' => $this->t('Telephone'),
        'email' => $this->t('Email'),
        'url' => $this->t('Url'),
        'week' => $this->t('Week'),
        'month' => $this->t('Month'),
        'time' => $this->t('Time'),
        'date' => $this->t('Date'),
        'datetime-local' => $this->t('Datetime local'),
      ],
      '#default_value' => $this->getSetting('type'),
    ];
    $element['placeholder'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Placeholder'),
      '#default_value' => $this->getSetting('placeholder'),
      '#description' => implode('<br/>', [
        $this->t('Text that will be shown inside the field until a value is entered. This hint is usually a sample value or a brief description of the expected format.'),
        $this->t('Email @pattern', ['@pattern' => 'mail@company.com']),
        $this->t('Telephone @pattern', ['@pattern' => '01 23 45 67 89']),
      ]),
    ];
    $element['pattern'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Regex pattern for input'),
      '#default_value' => $this->getSetting('pattern'),
      '#description' => implode('<br/>', [
        $this->t('Email @pattern', ['@pattern' => '[^@\s]+@[^@\s]+']),
        $this->t('Telephone @pattern', ['@pattern' => '^(\d{2} ?){4}\d{2}$']),
      ]),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'textfield'],
            ['value' => 'number'],
            ['value' => 'search'],
            ['value' => 'date'],
            ['value' => 'email'],
            ['value' => 'tel'],
            ['value' => 'url'],
          ],
        ],
      ],
    ];
    $element['separator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Separator'),
      '#default_value' => $this->getSetting('separator'),
      '#description' => $this->t('Specify a character to automatically separate large numbers.'),
      '#attributes' => [
        'maxlength' => 1,
        'size' => 1,
      ],
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'textfield'],
            ['value' => 'number'],
            ['value' => 'search'],
            ['value' => 'tel'],
          ],
        ],
      ],
    ];
    $element['step'] = [
      '#type' => 'number',
      '#title' => $this->t('Step', [], ['context' => 'numeric item']),
      '#step' => $base_step,
      '#default_value' => $this->getSetting('step'),
      '#description' => implode('<br/>', [
        $this->t('The minimum allowed amount to increment or decrement the field value. Note that setting an integer for this value on a decimal or float field restricts the input on the field to integer values only. While updating this field it is recommended to keep the <em>Default Value</em>, <em>Minimum</em> and <em>Maximum</em> fields blank. Leave blank for default = <em>@default</em>.', [
          '@default' => $field_settings['step'] ?? $base_step ?? '',
        ]),
        $this->t('Type telephone use with separator'),
      ]),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'tel'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['min'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum', [], ['context' => 'numeric item']),
      '#default_value' => $this->getSetting('min'),
      '#field_suffix' => "min $min, max $max",
      '#description' => $this->t('The minimum value that should be allowed in this field. Leave blank for default = <em>@default</em>.', [
        '@default' => $field_settings['min'] ?? $min ?? '',
      ]),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['max'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum', [], ['context' => 'numeric item']),
      '#default_value' => $this->getSetting('max'),
      '#field_suffix' => "min $min, max $max",
      '#description' => $this->t('The maximum value that should be allowed in this field. Leave blank for default = <em>@default</em>.', [
        '@default' => $field_settings['max'] ?? $max ?? '',
      ]),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'number'],
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['maxlength'] = [
      '#type' => 'number',
      '#title' => $this->t('Max length'),
      '#step' => 1,
      '#default_value' => $this->getSetting('maxlength'),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'range'],
            ['value' => 'week'],
            ['value' => 'month'],
            ['value' => 'time'],
            ['value' => 'date'],
            ['value' => 'datetime-local'],
          ],
        ],
      ],
    ];
    $element['size'] = [
      '#type' => 'number',
      '#title' => $this->t('Size'),
      '#step' => 1,
      '#default_value' => $this->getSetting('size'),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'tel'],
            ['value' => 'url'],
            ['value' => 'email'],
            ['value' => 'search'],
            ['value' => 'password'],
            ['value' => 'textfield'],
          ],
        ],
      ],
    ];
    $element['list'] = [
      '#type' => 'textfield',
      '#title' => $this->t('list'),
      '#step' => 1,
      '#default_value' => $this->getSetting('list'),
    ];
    $element['class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom class'),
      '#default_value' => $this->getSetting('class'),
    ];
    $element['custom'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom attributes'),
      '#description' => $this->t('Each variable is a new line. Can use with twig. Example: <br/>data-bs-toggle="tooltip"<br/>data-bs-title="{{entity.title.value}}"'),
      '#default_value' => $this->getSetting('custom'),
    ];
    $element['use_auto_numeric'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use auto numeric'),
      '#default_value' => $this->getSetting('use_auto_numeric'),
      '#states' => [
        'visible' => [
          ":input[name='{$field_state}[type]']" => [
            ['value' => 'number'],
          ],
        ],
      ],
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();
    foreach ($settings as $name => $value) {
      if (!empty($value)) {
        $summary[] = ucfirst($name) . ': ' . $value;
      }
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $settings = $this->getSettings();
    $field_settings = $this->fieldDefinition->getSettings();
    $field_type = $this->fieldDefinition->getType();
    $default_value = $items[$delta]->value ?? NULL;
    $languageId = $this->currentUser->getPreferredLangcode();
    $languageConfig = $this->getSeparatorLanguage($languageId);
    $element['value'] = $element + [
      '#type' => $settings['type'] == 'number' ? 'textfield' : $settings['type'],
      '#default_value' => $default_value,
      '#attributes' => [
        'class' => ['input-pattern'],
      ],
      '#attached' => [
        'library' => [
          'input_pattern/input_pattern',
        ],
      ],
    ];
    if (!empty($settings['use_auto_numeric']) && $settings['type'] == 'number') {
      $element['value']['#attributes']['autocomplete'] = 'off';
      $element['#attached']['drupalSettings']['auto_numeric']['language'] = $languageConfig['languageFormat'];
      $element['#attached']['library'][] = 'input_pattern/auto_numeric';
    }

    if (!empty($settings['class'])) {
      $element['value']['#attributes']['class'][] = $settings['class'];
      unset($settings['class']);
    }
    $native = ['size', 'maxlength', 'placeholder'];
    foreach ($native as $attribute) {
      if (!empty($settings[$attribute])) {
        $element['value']['#' . $attribute] = $settings[$attribute];
      }
      if (isset($settings[$attribute])) {
        unset($settings[$attribute]);
      }
    }
    if (!empty($settings['custom'])) {
      $attributes = explode(PHP_EOL, $settings['custom']);
      $context = [
        'form' => $form,
        'delta' => $delta,
        'entity' => $items->getEntity() ?? '',
        'values' => $form_state->getValues() ?? [],
        'field_name' => $items->getName(),
      ];
      foreach ($attributes as $attribute) {
        if (empty(trim($attribute))) {
          continue;
        }
        $variables = explode('=', $attribute);
        $key = trim(str_replace(['"', "'", ' '], ['', '', '-'], $variables[0]));
        $val = trim(str_replace(['"', "'"], '', $variables[1] ?? $variables[0]));
        $settings[$key] = $this->twig->renderInline($val, $context);
      }
      unset($settings['custom']);
    }

    if (in_array($field_type, ['integer', 'float', 'decimal'])) {
      $value_field_primitive = $this->getLimitNumber($field_settings, $field_type);
      $min = $value_field_primitive && is_numeric($value_field_primitive['min']) ? $value_field_primitive['min'] : NULL;
      $max = $value_field_primitive && is_numeric($value_field_primitive['max']) ? $value_field_primitive['max'] : NULL;
      if ($settings['max'] == '') {
        $settings['max'] = is_numeric($field_settings['max']) ? $field_settings['max'] : $max;
      }
      if ($settings['min'] == '') {
        $settings['min'] = is_numeric($field_settings['min']) ? $field_settings['min'] : $min;
      }
    }
    foreach ($settings as $name => $value) {
      if ($name == 'use_auto_numeric' && empty($value)) {
        continue;
      }
      if ($value !== '' && $value !== NULL) {
        $element['value']['#attributes'][$name] = $value;
      }
    }

    if (!empty($field_settings['prefix'])) {
      $element['value']['#field_prefix'] = FieldFilteredMarkup::create($field_settings['prefix']);
    }
    if (!empty($field_settings['suffix'])) {
      $element['value']['#field_suffix'] = FieldFilteredMarkup::create($field_settings['suffix']);
    }
    $element['value']['#attributes']['field_type'] = $field_type;
    if ($field_type == 'decimal') {
      $element['value']['#attributes']['scale'] = $field_settings['scale'];
    }
    $this->extractValues($form, $form_state);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    $this->extractValues($form, $form_state);
    parent::extractFormValues($items, $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();
    $field_settings = $this->getFieldSettings();
    $field_type = $this->fieldDefinition->getFieldStorageDefinition()->getType();
    foreach ($values as &$value) {
      if (!empty($value['value'])) {
        $limit = $this->getLimitNumber($field_settings, $field_type);
        if (empty($limit)) {
          continue;
        }
        $min = $settings['min'] !== '' ? (int) $settings['min'] : (is_numeric($field_settings['min']) ? $field_settings['min'] : $limit['min']);
        $max = $settings['max'] !== '' ? (int) $settings['max'] : (is_numeric($field_settings['max']) ? $field_settings['max'] : $limit['max']);
        if (in_array($field_type, ['integer', 'decimal', 'bigint', 'float']) && is_numeric($value['value'])) {
          if ($limit && ($value['value'] < $min || $value['value'] > $max)) {
            if ($value['value'] < $min) {
              $value['value'] = $min;
            }
            else {
              $value['value'] = $max;
            }
            $message = implode(' ', [
              $this->fieldDefinition->getLabel(),
              $this->t('Numeric value out of range'),
              $value['value'],
            ]);
            $this->messenger()->addError($message);
          }
        }
      }
    }
    return parent::massageFormValues($values, $form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getLimitNumber($field_settings, $field_type) {
    $limit = FALSE;
    if (!empty($field_settings["size"])) {
      $limit = match ($field_settings["size"]) {
        'normal' => ['min' => -2147483648, 'max' => 2147483647],
        'big' => ['min' => -9223372036854775808, 'max' => 9223372036854775807],
      };
    }
    if (in_array($field_type, [
      'decimal',
      'float',
    ]) && !empty($field_settings['precision']) && !empty($field_settings['scale'])) {
      $value_allow = str_repeat('9', $field_settings['precision'] - $field_settings['scale']) . '.' . str_repeat('9', $field_settings['scale']);
      $limit = [
        'min' => -$value_allow,
        'max' => $value_allow,
      ];
    }
    return $limit;
  }

  /**
   * {@inheritdoc}
   */
  protected function getSeparatorLanguage($language_code) {
    $language_format = 'British';
    $formatNum = [
      'French' => [
        'fr',
        'af',
        'ar',
        'be',
        'bg',
        'br',
        'cs',
        'eo',
        'et',
        'fi',
        'ht',
        'hu',
        'hy',
        'ka',
        'kk',
        'ky',
        'lt',
        'lv',
        'mg',
        'nb',
        'nn',
        'oc',
        'os',
        'pl',
        'prs',
        'ps',
        'pt',
        'pt-pt',
        'ru',
        'rw',
        'se',
        'sk',
        'sq',
        'sv',
        'tyv',
        'uk',
      ],
      'Spanish' => [
        'de',
        'ast',
        'az',
        'bs',
        'ca',
        'da',
        'el',
        'es',
        'eu',
        'fo',
        'fy',
        'gl',
        'hr',
        'id',
        'is',
        'it',
        'jv',
        'ku',
        'mk',
        'nl',
        'pt-br',
        'ro',
        'sl',
        'sr',
        'tr',
        'vi',
      ],
    ];
    foreach ($formatNum as $lang => $languageIso) {
      if (in_array($language_code, $languageIso)) {
        $language_format = $lang;
        break;
      }
    }
    $mapSeparator = [
      'British' => ['separator' => ',', 'decimalCharacter' => '.'],
      'French' => ['separator' => '.', 'decimalCharacter' => ','],
      'Spanish' => ['separator' => '.', 'decimalCharacter' => ','],
    ];
    return $mapSeparator[$language_format] + ['languageFormat' => $language_format];
  }

  /**
   * {@inheritdoc}
   */
  private function extractValues(array &$form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();
    $field_name_path = $field_name;
    $userInput = $form_state->getUserInput();
    if ($form['#parents']) {
      $field_name_path = $form['#parents'];
      $field_name_path[] = $field_name;
    }
    if (is_array($field_name_path)) {
      $user_input = NestedArray::getValue($userInput, $field_name_path, $exists);
    }
    else {
      $user_input = $userInput[$field_name_path] ?? [];
    }

    if (!empty($user_input)) {
      $settings = $this->getSettings();
      $languageId = $this->currentUser->getPreferredLangcode();
      $languageConfig = $this->getSeparatorLanguage($languageId);
      $separator = $settings['separator'];
      $decimal_character = $settings['separator'] == '.' ? ',' : '.';
      if (!empty($settings['use_auto_numeric'])) {
        $separator = $languageConfig['separator'];
        $decimal_character = $languageConfig['decimalCharacter'];
      }

      $field_value = $form_state->getValue($field_name_path) ?? [];
      foreach ($user_input as $delta => &$value) {
        if (is_numeric($delta) && gettype($value['value']) != 'double' && $value['value'] != '') {
          $value['value'] = str_replace($separator, '', $value['value']);
          $value['value'] = str_replace($decimal_character, '.', $value['value']);
          $value['value'] += 0;
          $field_value[$delta]['value'] = $value['value'];
        }
      }
      if (is_array($field_name_path)) {
        NestedArray::setValue($userInput, $field_name_path, $user_input);
      }
      else {
        $userInput[$field_name] = $user_input;
      }
      $form_state->setValue($field_name_path, $field_value);
      $form_state->setUserInput($userInput);
    }
  }

}
